@@ -16,11 +16,13 @@
 #ifndef ACCOUNTINFO_H
 #define ACCOUNTINFO_H
 
+#include "connectionvalidator.h"
+#include "creds/abstractcredentials.h"
+
 #include <QByteArray>
 #include <QElapsedTimer>
 #include <QPointer>
-#include "connectionvalidator.h"
-#include "creds/abstractcredentials.h"
+#include <QTimer>
 
 #include <memory>
 
@@ -175,6 +177,8 @@ public:
     
     void trySignIn();
 
+    void systemOnlineConfigurationChanged();
+
 public slots:
     /// Triggers a ping to the server to update state and
     /// connection status and errors.
@@ -184,6 +188,10 @@ private:
     void setState(State state);
     void fetchNavigationApps();
 
+    int retryCount() const;
+    void increaseRetryCount();
+    void resetRetryCount();
+
 signals:
     void stateChanged(State state);
     void isConnectedChanged();
@@ -205,6 +213,12 @@ protected Q_SLOTS:
     void slotEtagResponseHeaderReceived(const QByteArray &value, int statusCode);
     void slotOcsError(int statusCode, const QString &message);
 
+private Q_SLOTS:
+
+    void slotCheckConnection();
+    void slotPushNotificationsReady();
+    void slotServerUserStatusChanged();
+
 private:
     AccountPtr _account;
     State _state;
@@ -241,6 +255,12 @@ private:
     AccountAppList _apps;
 
     bool _isDesktopNotificationsAllowed;
+
+    int _retryCount = 0;
+
+    QTimer _checkConnectionTimer;
+    QElapsedTimer _lastCheckConnectionTimer;
+
 };
 
 class AccountApp : public QObject
