@@ -46,6 +46,7 @@ Q_DECLARE_LOGGING_CATEGORY(lcApplication)
 
 class Theme;
 class Folder;
+class ShellExtensionsServer;
 class SslErrorDialog;
 
 /**
@@ -71,6 +72,8 @@ public:
 
     ownCloudGui *gui() const;
 
+    bool event(QEvent *event) override;
+
 public slots:
     // TODO: this should not be public
     void slotownCloudWizardDone(int);
@@ -84,11 +87,12 @@ public slots:
     /// Attempt to show() the tray icon again. Used if no systray was available initially.
     void tryTrayAgain();
 
+    void handleEditLocally(const QUrl &url) const;
+
 protected:
     void parseOptions(const QStringList &);
     void setupTranslations();
     void setupLogging();
-    bool event(QEvent *event) override;
 
 signals:
     void folderRemoved();
@@ -108,6 +112,8 @@ protected slots:
 private:
     void setHelp();
 
+    void handleEditLocallyFromOptions();
+
     /**
      * Maybe a newer version of the client was used with this config file:
      * if so, backup, confirm with user and remove the config that can't be read.
@@ -134,6 +140,7 @@ private:
     bool _userTriggeredConnect;
     bool _debugMode;
     bool _backgroundMode;
+    QUrl _editFileLocallyUrl;
 
     ClientProxy _proxy;
 
@@ -144,6 +151,9 @@ private:
     QScopedPointer<CrashReporter::Handler> _crashHandler;
 #endif
     QScopedPointer<FolderMan> _folderManager;
+#ifdef Q_OS_WIN
+    QScopedPointer<ShellExtensionsServer> _shellExtensionsServer;
+#endif
 };
 
 } // namespace OCC
