@@ -150,6 +150,7 @@ void WebFlowCredentials::askFromUser() {
 #ifdef WITH_WEBENGINE
         bool useFlow2 = (type != DetermineAuthTypeJob::WebViewFlow);
 #else // WITH_WEBENGINE
+        Q_UNUSED(type)
         bool useFlow2 = true;
 #endif // WITH_WEBENGINE
 
@@ -162,8 +163,8 @@ void WebFlowCredentials::askFromUser() {
             _askDialog->setUrl(url);
         }
 
-        QString msg = tr("You have been logged out of %1 as user %2. Please login again.")
-                          .arg(_account->displayName(), _user);
+        QString msg = tr("You have been logged out of your account %1 at %2. Please login again.")
+                          .arg(_user, _account->displayName());
         _askDialog->setInfo(msg);
 
         _askDialog->show();
@@ -186,7 +187,7 @@ void WebFlowCredentials::slotAskFromUserCredentialsProvided(const QString &user,
     } else {
         qCInfo(lcWebFlowCredentials()) << "Authed with the wrong user!";
 
-        QString msg = tr("Please login with the user: %1")
+        QString msg = tr("Please login with the account: %1")
                 .arg(_user);
         _askDialog->setError(msg);
 
