@@ -117,7 +117,7 @@ public:
      */
     Folder(const FolderDefinition &definition, AccountState *accountState, std::unique_ptr<Vfs> vfs, QObject *parent = nullptr);
 
-    ~Folder();
+    ~Folder() override;
 
     using Map = QMap<QString, Folder *>;
     using MapIterator = QMapIterator<QString, Folder *>;
@@ -256,6 +256,8 @@ public:
       */
     void scheduleThisFolderSoon();
 
+    void acceptInvalidFileName(const QString &filePath);
+
     /**
       * Migration: When this flag is true, this folder will save to
       * the backwards-compatible 'Folders' section in the config file.
@@ -287,9 +289,15 @@ public:
     bool isVfsOnOffSwitchPending() const { return _vfsOnOffPending; }
     void setVfsOnOffSwitchPending(bool pending) { _vfsOnOffPending = pending; }
 
+    void switchToVirtualFiles();
+
+    void processSwitchedToVirtualFiles();
+
     /** Whether this folder should show selective sync ui */
     bool supportsSelectiveSync() const;
 
+    QString fileFromLocalPath(const QString &localPath) const;
+
 signals:
     void syncStateChange();
     void syncStarted();
@@ -370,12 +378,14 @@ private slots:
      */
     void slotSyncError(const QString &message, ErrorCategory category = ErrorCategory::Normal);
 
+    void slotAddErrorToGui(SyncFileItem::Status status, const QString &errorMessage, const QString &subject = {});
+
     void slotTransmissionProgress(const ProgressInfo &pi);
     void slotItemCompleted(const SyncFileItemPtr &);
 
     void slotRunEtagJob();
-    void etagRetrieved(const QString &, const QDateTime &tp);
-    void etagRetrievedFromSyncEngine(const QString &, const QDateTime &time);
+    void etagRetrieved(const QByteArray &, const QDateTime &tp);
+    void etagRetrievedFromSyncEngine(const QByteArray &, const QDateTime &time);
 
     void slotEmitFinishedDelayed();
 
@@ -420,7 +430,7 @@ private:
 
     void checkLocalPath();
 
-    void setSyncOptions();
+    SyncOptions initializeSyncOptions() const;
 
     enum LogStatus {
         LogStatusRemove,
@@ -438,6 +448,8 @@ private:
 
     void startVfs();
 
+    void correctPlaceholderFiles();
+
     AccountStatePtr _accountState;
     FolderDefinition _definition;
     QString _canonicalLocalPath; // As returned with QFileInfo:canonicalFilePath.  Always ends with "/"
@@ -445,7 +457,7 @@ private:
     SyncResult _syncResult;
     QScopedPointer<SyncEngine> _engine;
     QPointer<RequestEtagJob> _requestEtagJob;
-    QString _lastEtag;
+    QByteArray _lastEtag;
     QElapsedTimer _timeSinceLastSyncDone;
     QElapsedTimer _timeSinceLastSyncStart;
     QElapsedTimer _timeSinceLastFullLocalDiscovery;
@@ -492,6 +504,10 @@ private:
      */
     bool _vfsOnOffPending = false;
 
+    /** Whether this folder has just switched to VFS or not
+     */
+    bool _hasSwitchedToVfs = false;
+
     /**
      * Watches this folder's local directory for changes.
      *
