@@ -26,6 +26,7 @@
 #include "syncfileitem.h"
 
 class TestFolderMan;
+class TestCfApiShellExtensionsIPC;
 
 namespace OCC {
 
@@ -62,7 +63,7 @@ class FolderMan : public QObject
 {
     Q_OBJECT
 public:
-    ~FolderMan();
+    ~FolderMan() override;
     static FolderMan *instance();
 
     int setupFolders();
@@ -201,6 +202,9 @@ public:
     void setDirtyProxy();
     void setDirtyNetworkLimits();
 
+    /** opens a file with default app, if the file is present **/
+    void editFileLocally(const QString &accountDisplayName, const QString &relPath);
+
 signals:
     /**
       * signal to indicate a folder has changed its sync state.
@@ -324,6 +328,8 @@ private:
 
     bool pushNotificationsFilesReady(Account *account);
 
+    bool isSwitchToVfsNeeded(const FolderDefinition &folderDefinition) const;
+
     QSet<Folder *> _disabledFolders;
     Folder::Map _folderMap;
     QString _folderConfigPath;
@@ -360,6 +366,7 @@ private:
     explicit FolderMan(QObject *parent = nullptr);
     friend class OCC::Application;
     friend class ::TestFolderMan;
+    friend class ::TestCfApiShellExtensionsIPC;
 };
 
 } // namespace OCC
