@@ -36,8 +36,6 @@
 #include "common/utility.h"
 #include "logger.h"
 
-#include "config.h"
-
 #include "legalnotice.h"
 
 #include <QFileDialog>
@@ -116,7 +114,9 @@ void createDebugArchive(const QString &filename)
 {
     const auto entries = createFileList();
 
+    // TODO: Port away from this private API (best to port to KArchive)
     QZipWriter zip(filename);
+    zip.setCreationPermissions(zip.creationPermissions() | QFile::ReadOther);
     for (const auto &entry : entries) {
         if (entry.localFilename.isEmpty()) {
             zip.addDirectory(entry.zipFilename);
@@ -148,6 +148,10 @@ GeneralSettings::GeneralSettings(QWidget *parent)
         this, &GeneralSettings::slotToggleOptionalServerNotifications);
     _ui->serverNotificationsCheckBox->setToolTip(tr("Server notifications that require attention."));
 
+    connect(_ui->callNotificationsCheckBox, &QAbstractButton::toggled,
+        this, &GeneralSettings::slotToggleCallNotifications);
+    _ui->callNotificationsCheckBox->setToolTip(tr("Show call notification dialogs."));
+
     connect(_ui->showInExplorerNavigationPaneCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotShowInExplorerNavigationPane);
 
     // Rename 'Explorer' appropriately on non-Windows
@@ -249,6 +253,8 @@ void GeneralSettings::loadMiscSettings()
     ConfigFile cfgFile;
     _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
     _ui->serverNotificationsCheckBox->setChecked(cfgFile.optionalServerNotifications());
+    _ui->callNotificationsCheckBox->setEnabled(_ui->serverNotificationsCheckBox->isEnabled());
+    _ui->callNotificationsCheckBox->setChecked(cfgFile.showCallNotifications());
     _ui->showInExplorerNavigationPaneCheckBox->setChecked(cfgFile.showInExplorerNavigationPane());
     _ui->crashreporterCheckBox->setChecked(cfgFile.crashReporter());
     auto newFolderLimit = cfgFile.newBigFolderSizeLimit();
@@ -297,6 +303,7 @@ void GeneralSettings::slotUpdateInfo()
     else if (auto sparkleUpdater = qobject_cast<SparkleUpdater *>(Updater::instance())) {
         _ui->updateStateLabel->setText(sparkleUpdater->statusString());
         _ui->restartButton->setVisible(false);
+        connect(_ui->updateButton, &QAbstractButton::clicked, this, &GeneralSettings::slotUpdateCheckNow, Qt::UniqueConnection);
     }
 #endif
 
@@ -306,10 +313,43 @@ void GeneralSettings::slotUpdateInfo()
         this, &GeneralSettings::slotUpdateChannelChanged, Qt::UniqueConnection);
 }
 
-void GeneralSettings::slotUpdateChannelChanged(const QString &channel)
+void GeneralSettings::slotUpdateChannelChanged(const QString &translatedChannel)
 {
-    if (channel == ConfigFile().updateChannel())
+    const auto updateChannelToLocalized = [](const QString &channel) {
+        auto decodedTranslatedChannel = QString{};
+
+        if (channel == QStringLiteral("stable")) {
+            decodedTranslatedChannel = tr("stable");
+        } else if (channel == QStringLiteral("beta")) {
+            decodedTranslatedChannel = tr("beta");
+        }
+
+        return decodedTranslatedChannel;
+    };
+
+    const auto updateChannelFromLocalized = [](const int index) {
+        auto channel = QString{};
+
+        switch (index)
+        {
+        case 0:
+            channel = QStringLiteral("stable");
+            break;
+        case 1:
+            channel = QStringLiteral("beta");
+            break;
+        default:
+            channel = QString{};
+        }
+
+        return channel;
+    };
+
+    const auto channel = updateChannelFromLocalized(_ui->updateChannel->currentIndex());
+
+    if (translatedChannel == ConfigFile().updateChannel()) {
         return;
+    }
 
     auto msgBox = new QMessageBox(
         QMessageBox::Warning,
@@ -329,7 +369,7 @@ void GeneralSettings::slotUpdateChannelChanged(const QString &channel)
         this);
     auto acceptButton = msgBox->addButton(tr("Change update channel"), QMessageBox::AcceptRole);
     msgBox->addButton(tr("Cancel"), QMessageBox::RejectRole);
-    connect(msgBox, &QMessageBox::finished, msgBox, [this, channel, msgBox, acceptButton] {
+    connect(msgBox, &QMessageBox::finished, msgBox, [this, channel, msgBox, acceptButton, updateChannelToLocalized] {
         msgBox->deleteLater();
         if (msgBox->clickedButton() == acceptButton) {
             ConfigFile().setUpdateChannel(channel);
@@ -344,7 +384,7 @@ void GeneralSettings::slotUpdateChannelChanged(const QString &channel)
             }
 #endif
         } else {
-            _ui->updateChannel->setCurrentText(ConfigFile().updateChannel());
+            _ui->updateChannel->setCurrentText(updateChannelToLocalized(ConfigFile().updateChannel()));
         }
     });
     msgBox->open();
@@ -352,7 +392,11 @@ void GeneralSettings::slotUpdateChannelChanged(const QString &channel)
 
 void GeneralSettings::slotUpdateCheckNow()
 {
+#if defined(Q_OS_MAC) && defined(HAVE_SPARKLE)
+    auto *updater = qobject_cast<SparkleUpdater *>(Updater::instance());
+#else
     auto *updater = qobject_cast<OCUpdater *>(Updater::instance());
+#endif
     if (ConfigFile().skipUpdateCheck()) {
         updater = nullptr; // don't show update info if updates are disabled
     }
@@ -397,6 +441,13 @@ void GeneralSettings::slotToggleOptionalServerNotifications(bool enable)
 {
     ConfigFile cfgFile;
     cfgFile.setOptionalServerNotifications(enable);
+    _ui->callNotificationsCheckBox->setEnabled(enable);
+}
+
+void GeneralSettings::slotToggleCallNotifications(bool enable)
+{
+    ConfigFile cfgFile;
+    cfgFile.setShowCallNotifications(enable);
 }
 
 void GeneralSettings::slotShowInExplorerNavigationPane(bool checked)
