@@ -18,6 +18,7 @@
 #include "accountstate.h"
 #include "sharepermissions.h"
 #include "owncloudgui.h"
+#include "common/syncjournalfilerecord.h"
 
 #include <QSharedPointer>
 #include <QPointer>
@@ -26,6 +27,7 @@
 #include <QWidget>
 
 class QProgressIndicator;
+class QVBoxLayout;
 
 namespace OCC {
 
@@ -34,6 +36,7 @@ namespace Ui {
 }
 
 class ShareLinkWidget;
+class InternalLinkWidget;
 class ShareUserGroupWidget;
 class ShareManager;
 class LinkShare;
@@ -49,6 +52,7 @@ public:
         const QString &localPath,
         SharePermissions maxSharingPermissions,
         const QByteArray &numericFileId,
+        SyncJournalFileLockInfo filelockState,
         ShareDialogStartPage startPage,
         QWidget *parent = nullptr);
     ~ShareDialog() override;
@@ -67,7 +71,6 @@ private slots:
     void slotCreatePasswordForLinkShare(const QString &password);
     void slotCreatePasswordForLinkShareProcessed();
     void slotLinkShareRequiresPassword(const QString &message);
-    void slotAdjustScrollWidgetSize();
 
 signals:
     void toggleShareLinkAnimation(bool start);
@@ -75,12 +78,14 @@ signals:
 
 protected:
     void changeEvent(QEvent *) override;
+    void resizeEvent(QResizeEvent *event) override;
 
 private:
     void showSharingUi();
     void initShareManager();
     ShareLinkWidget *addLinkShareWidget(const QSharedPointer<LinkShare> &linkShare);
     void initLinkShareWidget();
+    void adjustScrollWidget();
 
     Ui::ShareDialog *_ui;
 
@@ -89,14 +94,19 @@ private:
     QString _localPath;
     SharePermissions _maxSharingPermissions;
     QByteArray _numericFileId;
+    SyncJournalFileLockInfo _filelockState;
     QString _privateLinkUrl;
     ShareDialogStartPage _startPage;
     ShareManager *_manager = nullptr;
 
     QList<ShareLinkWidget*> _linkWidgetList;
     ShareLinkWidget* _emptyShareLinkWidget = nullptr;
+    InternalLinkWidget* _internalLinkWidget = nullptr;
     ShareUserGroupWidget *_userGroupWidget = nullptr;
     QProgressIndicator *_progressIndicator = nullptr;
+    
+    QWidget *_scrollAreaViewPort = nullptr;
+    QVBoxLayout *_scrollAreaLayout = nullptr;
 };
 
 } // namespace OCC
