@@ -82,13 +82,10 @@ ShareLinkWidget::ShareLinkWidget(AccountPtr account,
 
     _ui->errorLabel->hide();
 
-    auto sharingPossible = true;
     if (!_account->capabilities().sharePublicLink()) {
         qCWarning(lcShareLink) << "Link shares have been disabled";
-        sharingPossible = false;
     } else if (!(maxSharingPermissions & SharePermissionShare)) {
         qCWarning(lcShareLink) << "The file can not be shared because it was shared without sharing permission.";
-        sharingPossible = false;
     }
 
     _ui->enableShareLink->setChecked(false);
@@ -301,13 +298,13 @@ void ShareLinkWidget::setupUiOptions()
     }
 
     // Adds action to unshare widget (check box)
-    _unshareLinkAction = _linkContextMenu->addAction(QIcon(":/client/theme/delete.svg"),
-        tr("Delete link"));
+    _unshareLinkAction.reset(_linkContextMenu->addAction(QIcon(":/client/theme/delete.svg"),
+        tr("Delete link")));
 
     _linkContextMenu->addSeparator();
 
-    _addAnotherLinkAction = _linkContextMenu->addAction(QIcon(":/client/theme/add.svg"),
-        tr("Add another link"));
+    _addAnotherLinkAction.reset(_linkContextMenu->addAction(QIcon(":/client/theme/add.svg"),
+        tr("Add another link")));
 
     _ui->enableShareLink->setIcon(QIcon(":/client/theme/copy.svg"));
     disconnect(_ui->enableShareLink, &QPushButton::clicked, this, &ShareLinkWidget::slotCreateShareLink);
@@ -411,22 +408,6 @@ void ShareLinkWidget::slotPasswordSetError(const int code, const QString &messag
     emit createPasswordProcessed();
 }
 
-void ShareLinkWidget::startAnimation(const int start, const int end)
-{
-    auto *animation = new QPropertyAnimation(this, "maximumHeight", this);
-
-    animation->setDuration(500);
-    animation->setStartValue(start);
-    animation->setEndValue(end);
-
-    connect(animation, &QAbstractAnimation::finished, this, &ShareLinkWidget::slotAnimationFinished);
-    if (end < start) // that is to remove the widget, not to show it
-        connect(animation, &QAbstractAnimation::finished, this, &ShareLinkWidget::slotDeleteAnimationFinished);
-    connect(animation, &QVariantAnimation::valueChanged, this, &ShareLinkWidget::resizeRequested);
-
-    animation->start();
-}
-
 void ShareLinkWidget::slotDeleteShareFetched()
 {
     slotToggleShareLinkAnimation(false);
@@ -450,12 +431,6 @@ void ShareLinkWidget::toggleNoteOptions(const bool enable)
     } 
 }
 
-void ShareLinkWidget::slotAnimationFinished()
-{
-    emit resizeRequested();
-    deleteLater();
-}
-
 void ShareLinkWidget::slotCreateLabel()
 {
     const auto labelText = _shareLinkEdit->text();
@@ -474,14 +449,6 @@ void ShareLinkWidget::slotLabelSet()
     displayShareLinkLabel();
 }
 
-void ShareLinkWidget::slotDeleteAnimationFinished()
-{
-    // There is a painting bug where a small line of this widget isn't
-    // properly cleared. This explicit repaint() call makes sure any trace of
-    // the share widget is removed once it's destroyed. #4189
-    connect(this, SIGNAL(destroyed(QObject *)), parentWidget(), SLOT(repaint()));
-}
-
 void ShareLinkWidget::slotCreateShareRequiresPassword(const QString &message)
 {
     slotToggleShareLinkAnimation(message.isEmpty());
@@ -517,8 +484,11 @@ void ShareLinkWidget::toggleExpireDateOptions(const bool enable)
     const auto date = enable ? _linkShare->getExpireDate() : QDate::currentDate().addDays(1);
     _ui->calendar->setDate(date);
     _ui->calendar->setMinimumDate(QDate::currentDate().addDays(1));
-    _ui->calendar->setMaximumDate(
-        QDate::currentDate().addDays(_account->capabilities().sharePublicLinkExpireDateDays()));
+
+    if(_account->capabilities().sharePublicLinkExpireDateDays() > 0) {
+        _ui->calendar->setMaximumDate(QDate::currentDate().addDays(_account->capabilities().sharePublicLinkExpireDateDays()));
+    }
+
     _ui->calendar->setFocus();
     
     if (!enable && _linkShare && _linkShare->getExpireDate().isValid()) {
@@ -570,7 +540,7 @@ void ShareLinkWidget::slotLinkContextMenuActionTriggered(QAction *action)
     const auto state = action->isChecked();
     SharePermissions perm = SharePermissionRead;
 
-    if (action == _addAnotherLinkAction) {
+    if (action == _addAnotherLinkAction.data()) {
         emit createLinkShare();
 
     } else if (action == _readOnlyLinkAction && state) {
@@ -597,7 +567,7 @@ void ShareLinkWidget::slotLinkContextMenuActionTriggered(QAction *action)
     } else if (action == _noteLinkAction) {
         toggleNoteOptions(state);
 
-    } else if (action == _unshareLinkAction) {
+    } else if (action == _unshareLinkAction.data()) {
         confirmAndDeleteShare();
     }
 }
@@ -623,9 +593,13 @@ void ShareLinkWidget::slotStyleChanged()
 
 void ShareLinkWidget::customizeStyle()
 {
-    _unshareLinkAction->setIcon(Theme::createColorAwareIcon(":/client/theme/delete.svg"));
+    if(_unshareLinkAction) {
+        _unshareLinkAction->setIcon(Theme::createColorAwareIcon(":/client/theme/delete.svg"));
+    }
 
-    _addAnotherLinkAction->setIcon(Theme::createColorAwareIcon(":/client/theme/add.svg"));
+    if(_addAnotherLinkAction) {
+        _addAnotherLinkAction->setIcon(Theme::createColorAwareIcon(":/client/theme/add.svg"));
+    }
 
     _ui->enableShareLink->setIcon(Theme::createColorAwareIcon(":/client/theme/copy.svg"));
 
