@@ -27,6 +27,7 @@ class QQmlApplicationEngine;
 class QQuickWindow;
 class QWindow;
 class QQuickWindow;
+class QGuiApplication;
 
 namespace OCC {
 
@@ -38,10 +39,20 @@ public:
     QNetworkAccessManager* create(QObject *parent) override;
 };
 
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
+enum MacNotificationAuthorizationOptions {
+    Default = 0,
+    Provisional
+};
+
+void setUserNotificationCenterDelegate();
+void checkNotificationAuth(MacNotificationAuthorizationOptions authOptions = MacNotificationAuthorizationOptions::Provisional);
+void registerNotificationCategories(const QString &localizedDownloadString);
 bool canOsXSendUserNotification();
 void sendOsXUserNotification(const QString &title, const QString &message);
+void sendOsXUpdateNotification(const QString &title, const QString &message, const QUrl &webUrl);
 void setTrayWindowLevelAndVisibleOnAllSpaces(QWindow *window);
+double statusBarThickness();
 #endif
 
 /**
@@ -55,6 +66,8 @@ class Systray
 
     Q_PROPERTY(QString windowTitle READ windowTitle CONSTANT)
     Q_PROPERTY(bool useNormalWindow READ useNormalWindow CONSTANT)
+    Q_PROPERTY(bool syncIsPaused READ syncIsPaused WRITE setSyncIsPaused NOTIFY syncIsPausedChanged)
+    Q_PROPERTY(bool isOpen READ isOpen WRITE setIsOpen NOTIFY isOpenChanged)
 
 public:
     static Systray *instance();
@@ -63,36 +76,53 @@ public:
     enum class TaskBarPosition { Bottom, Left, Top, Right };
     Q_ENUM(TaskBarPosition);
 
+    enum class NotificationPosition { Default, TopLeft, TopRight, BottomLeft, BottomRight };
+    Q_ENUM(NotificationPosition);
+
+    enum class WindowPosition { Default, Center };
+    Q_ENUM(WindowPosition);
+
     void setTrayEngine(QQmlApplicationEngine *trayEngine);
     void create();
     void showMessage(const QString &title, const QString &message, MessageIcon icon = Information);
+    void showUpdateMessage(const QString &title, const QString &message, const QUrl &webUrl);
     void setToolTip(const QString &tip);
-    bool isOpen();
-    QString windowTitle() const;
-    bool useNormalWindow() const;
+    void createCallDialog(const Activity &callNotification, const AccountStatePtr accountState);
 
-    Q_INVOKABLE void pauseResumeSync();
-    Q_INVOKABLE bool syncIsPaused();
-    Q_INVOKABLE void setOpened();
-    Q_INVOKABLE void setClosed();
-    Q_INVOKABLE void positionWindow(QQuickWindow *window) const;
-    Q_INVOKABLE void forceWindowInit(QQuickWindow *window) const;
+    Q_REQUIRED_RESULT QString windowTitle() const;
+    Q_REQUIRED_RESULT bool useNormalWindow() const;
+
+    Q_REQUIRED_RESULT bool syncIsPaused() const;
+    Q_REQUIRED_RESULT bool isOpen() const;
 
 signals:
     void currentUserChanged();
     void openAccountWizard();
-    void openMainDialog();
     void openSettings();
     void openHelp();
     void shutdown();
 
-    void hideWindow();
-    void showWindow();
     void openShareDialog(const QString &sharePath, const QString &localPath);
-    void showFileActivityDialog(const QString &sharePath, const QString &localPath);
+    void showFileActivityDialog(const QString &objectName, const int objectId);
+    void sendChatMessage(const QString &token, const QString &message, const QString &replyTo);
+    void showErrorMessageDialog(const QString &error);
+
+    void syncIsPausedChanged();
+    void isOpenChanged();
 
 public slots:
-    void slotNewUserSelected();
+    void slotCurrentUserChanged();
+
+    void forceWindowInit(QQuickWindow *window) const;
+    void positionWindowAtTray(QQuickWindow *window) const;
+    void positionWindowAtScreenCenter(QQuickWindow *window) const;
+    void positionNotificationWindow(QQuickWindow *window) const;
+
+    void showWindow(WindowPosition position = WindowPosition::Default);
+    void hideWindow();
+
+    void setSyncIsPaused(const bool syncIsPaused);
+    void setIsOpen(const bool isOpen);
 
 private slots:
     void slotUnpauseAllFolders();
@@ -104,19 +134,28 @@ private:
     static Systray *_instance;
     Systray();
 
+    void setupContextMenu();
+
     QScreen *currentScreen() const;
     QRect currentScreenRect() const;
     QPoint computeWindowReferencePoint() const;
+    QPoint computeNotificationReferencePoint(int spacing = 20, NotificationPosition position = NotificationPosition::Default) const;
     QPoint calcTrayIconCenter() const;
     TaskBarPosition taskbarOrientation() const;
     QRect taskbarGeometry() const;
+    QRect computeWindowRect(int spacing, const QPoint &topLeft, const QPoint &bottomRight) const;
     QPoint computeWindowPosition(int width, int height) const;
+    QPoint computeNotificationPosition(int width, int height, int spacing = 20, NotificationPosition position = NotificationPosition::Default) const;
 
     bool _isOpen = false;
     bool _syncIsPaused = true;
     QPointer<QQmlApplicationEngine> _trayEngine;
+    QPointer<QMenu> _contextMenu;
+    QSharedPointer<QQuickWindow> _trayWindow;
 
     AccessManagerFactory _accessManagerFactory;
+
+    QSet<qlonglong> _callsAlreadyNotified;
 };
 
 } // namespace OCC
