@@ -1,4 +1,4 @@
-/*
+﻿/*
  * Copyright (C) by Klaas Freitag <freitag@owncloud.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,8 @@
 
 #include "activitydata.h"
 #include "activitylistmodel.h"
+#include "systray.h"
+#include "tray/usermodel.h"
 
 #include "theme.h"
 
@@ -54,22 +56,35 @@ ActivityListModel::ActivityListModel(AccountState *accountState,
 
 QHash<int, QByteArray> ActivityListModel::roleNames() const
 {
-    QHash<int, QByteArray> roles;
+    auto roles = QAbstractListModel::roleNames();
     roles[DisplayPathRole] = "displayPath";
     roles[PathRole] = "path";
-    roles[AbsolutePathRole] = "absolutePath";
+    roles[OpenablePathRole] = "openablePath";
     roles[DisplayLocationRole] = "displayLocation";
     roles[LinkRole] = "link";
     roles[MessageRole] = "message";
     roles[ActionRole] = "type";
-    roles[ActionIconRole] = "icon";
+    roles[DarkIconRole] = "darkIcon";
+    roles[LightIconRole] = "lightIcon";
     roles[ActionTextRole] = "subject";
     roles[ActionsLinksRole] = "links";
+    roles[ActionsLinksContextMenuRole] = "linksContextMenu";
+    roles[ActionsLinksForActionButtonsRole] = "linksForActionButtons";
     roles[ActionTextColorRole] = "activityTextTitleColor";
     roles[ObjectTypeRole] = "objectType";
+    roles[ObjectIdRole] = "objectId";
+    roles[ObjectNameRole] = "objectName";
     roles[PointInTimeRole] = "dateTime";
     roles[DisplayActions] = "displayActions";
     roles[ShareableRole] = "isShareable";
+    roles[IsCurrentUserFileActivityRole] = "isCurrentUserFileActivity";
+    roles[IsCurrentUserFileActivityRole] = "isCurrentUserFileActivity";
+    roles[ThumbnailRole] = "thumbnail";
+    roles[TalkNotificationConversationTokenRole] = "conversationToken";
+    roles[TalkNotificationMessageIdRole] = "messageId";
+    roles[TalkNotificationMessageSentRole] = "messageSent";
+    roles[TalkNotificationUserAvatarRole] = "userAvatar";
+
     return roles;
 }
 
@@ -78,9 +93,18 @@ void ActivityListModel::setAccountState(AccountState *state)
     _accountState = state;
 }
 
-void ActivityListModel::setCurrentlyFetching(bool value)
+void ActivityListModel::setCurrentItem(const int currentItem)
 {
+    _currentItem = currentItem;
+}
+
+void ActivityListModel::setAndRefreshCurrentlyFetching(bool value)
+{
+    if (_currentlyFetching == value) {
+        return;
+    }
     _currentlyFetching = value;
+    insertOrRemoveDummyFetchingActivity();
 }
 
 bool ActivityListModel::currentlyFetching() const
@@ -116,10 +140,11 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
         return QVariant();
 
     const auto getFilePath = [&]() {
-        if (!a._file.isEmpty()) {
+        const auto fileName = a._fileAction == QStringLiteral("file_renamed") ? a._renamedFile : a._file;
+        if (!fileName.isEmpty()) {
             const auto folder = FolderMan::instance()->folder(a._folder);
 
-            const QString relPath = folder ? folder->remotePath() + a._file : a._file;
+            const QString relPath = folder ? folder->remotePath() + fileName : fileName;
 
             const auto localFiles = FolderMan::instance()->findFileInLocalFolders(relPath, ast->account());
 
@@ -130,7 +155,7 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
             // If this is an E2EE file or folder, pretend we got no path, hiding the share button which is what we want
             if (folder) {
                 SyncJournalFileRecord rec;
-                folder->journalDb()->getFileRecord(a._file.mid(1), &rec);
+                folder->journalDb()->getFileRecord(fileName.mid(1), &rec);
                 if (rec.isValid() && (rec._isE2eEncrypted || !rec._e2eMangledName.isEmpty())) {
                     return QString();
                 }
@@ -165,13 +190,83 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
         return displayPath == "." || displayPath == "/" ? QString() : displayPath;
     };
 
+    const auto generatePreviewMap = [](const PreviewData &preview) {
+        return(QVariantMap {
+            {QStringLiteral("source"), QStringLiteral("image://tray-image-provider/").append(preview._source)},
+               {QStringLiteral("link"), preview._link},
+               {QStringLiteral("mimeType"), preview._mimeType},
+               {QStringLiteral("fileId"), preview._fileId},
+               {QStringLiteral("view"), preview._view},
+               {QStringLiteral("isMimeTypeIcon"), preview._isMimeTypeIcon},
+               {QStringLiteral("filename"), preview._filename},
+               {QStringLiteral("isUserAvatar"), false},
+        });
+    };
+
+    const auto generateAvatarThumbnailMap = [](const QString &avatarThumbnailUrl) {
+        return QVariantMap {
+            {QStringLiteral("source"), avatarThumbnailUrl},
+            {QStringLiteral("isMimeTypeIcon"), false},
+            {QStringLiteral("isUserAvatar"), true},
+        };
+    };
+
+    const auto generateIconPath = [&]() {
+        auto colorIconPath = role == DarkIconRole ? QStringLiteral("qrc:///client/theme/white/") : QStringLiteral("qrc:///client/theme/black/");
+        if (a._type == Activity::NotificationType && !a._talkNotificationData.userAvatar.isEmpty()) {
+            return QStringLiteral("qrc:///client/theme/colored/talk-bordered.svg");
+        } else if (a._type == Activity::SyncResultType) {
+            colorIconPath.append("state-error.svg");
+            return colorIconPath;
+        } else if (a._type == Activity::SyncFileItemType) {
+            if (a._status == SyncFileItem::NormalError
+                || a._status == SyncFileItem::FatalError
+                || a._status == SyncFileItem::DetailError
+                || a._status == SyncFileItem::BlacklistedError) {
+                colorIconPath.append("state-error.svg");
+                return colorIconPath;
+            } else if (a._status == SyncFileItem::SoftError
+                || a._status == SyncFileItem::Conflict
+                || a._status == SyncFileItem::Restoration
+                || a._status == SyncFileItem::FileLocked
+                || a._status == SyncFileItem::FileNameInvalid) {
+                colorIconPath.append("state-warning.svg");
+                return colorIconPath;
+            } else if (a._status == SyncFileItem::FileIgnored) {
+                colorIconPath.append("state-info.svg");
+                return colorIconPath;
+            } else {
+                // File sync successful
+                if (a._fileAction == "file_created") {
+                    return a._previews.empty() ? QStringLiteral("qrc:///client/theme/colored/add.svg")
+                                               : QStringLiteral("qrc:///client/theme/colored/add-bordered.svg");
+                } else if (a._fileAction == "file_deleted") {
+                    return a._previews.empty() ? QStringLiteral("qrc:///client/theme/colored/delete.svg")
+                                               : QStringLiteral("qrc:///client/theme/colored/delete-bordered.svg");
+                } else {
+                    return a._previews.empty() ? colorIconPath % QStringLiteral("change.svg")
+                                               : QStringLiteral("qrc:///client/theme/colored/change-bordered.svg");
+                }
+            }
+        } else {
+            // We have an activity
+            if (a._icon.isEmpty()) {
+                colorIconPath.append("activity.svg");
+                return colorIconPath;
+            }
+
+            const QString basePath = QStringLiteral("image://tray-image-provider/") % a._icon % QStringLiteral("/");
+            return role == DarkIconRole ? QString(basePath + QStringLiteral("white")) : QString(basePath + QStringLiteral("black"));
+        }
+    };
+
     switch (role) {
     case DisplayPathRole:
         return getDisplayPath();
     case PathRole:
-        return QUrl::fromLocalFile(QFileInfo(getFilePath()).path());
-    case AbsolutePathRole:
-        return getFilePath();
+        return QFileInfo(getFilePath()).path();
+    case OpenablePathRole:
+        return a._isMultiObjectActivity ? QFileInfo(getFilePath()).canonicalPath() : QFileInfo(getFilePath()).canonicalFilePath();
     case DisplayLocationRole:
         return displayLocation();
     case ActionsLinksRole: {
@@ -181,46 +276,24 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
         }
         return customList;
     }
-    case ActionIconRole: {
-        if (a._type == Activity::NotificationType) {
-            return "qrc:///client/theme/black/bell.svg";
-        } else if (a._type == Activity::SyncResultType) {
-            return "qrc:///client/theme/black/state-error.svg";
-        } else if (a._type == Activity::SyncFileItemType) {
-            if (a._status == SyncFileItem::NormalError
-                || a._status == SyncFileItem::FatalError
-                || a._status == SyncFileItem::DetailError
-                || a._status == SyncFileItem::BlacklistedError) {
-                return "qrc:///client/theme/black/state-error.svg";
-            } else if (a._status == SyncFileItem::SoftError
-                || a._status == SyncFileItem::Conflict
-                || a._status == SyncFileItem::Restoration
-                || a._status == SyncFileItem::FileLocked
-                || a._status == SyncFileItem::FileNameInvalid) {
-                return "qrc:///client/theme/black/state-warning.svg";
-            } else if (a._status == SyncFileItem::FileIgnored) {
-                return "qrc:///client/theme/black/state-info.svg";
-            } else {
-                // File sync successful
-                if (a._fileAction == "file_created") {
-                    return "qrc:///client/theme/colored/add.svg";
-                } else if (a._fileAction == "file_deleted") {
-                    return "qrc:///client/theme/colored/delete.svg";
-                } else {
-                    return "qrc:///client/theme/change.svg";
-                }
-            }
-        } else {
-            // We have an activity
-            if (a._icon.isEmpty()) {
-                return "qrc:///client/theme/black/activity.svg";
-            }
 
-            return a._icon;
-        }
+    case ActionsLinksContextMenuRole: {
+        return ActivityListModel::convertLinksToMenuEntries(a);
     }
+    
+    case ActionsLinksForActionButtonsRole: {
+        return ActivityListModel::convertLinksToActionButtons(a);
+    }
+
+    case DarkIconRole:
+    case LightIconRole:
+        return generateIconPath();
     case ObjectTypeRole:
         return a._objectType;
+    case ObjectIdRole:
+        return a._objectId;
+    case ObjectNameRole:
+        return a._objectName;
     case ActionRole: {
         switch (a._type) {
         case Activity::ActivityType:
@@ -249,7 +322,7 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
         if (a._link.isEmpty()) {
             return "";
         } else {
-            return a._link;
+            return a._link.toString();
         }
     }
     case AccountRole:
@@ -262,24 +335,50 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
     case DisplayActions:
         return _displayActions;
     case ShareableRole:
-        return !data(index, PathRole).toString().isEmpty() && _displayActions && a._fileAction != "file_deleted" && a._status != SyncFileItem::FileIgnored;
+        return !data(index, PathRole).toString().isEmpty() && a._objectType == QStringLiteral("files") && _displayActions && a._fileAction != "file_deleted" && a._status != SyncFileItem::FileIgnored;
+    case IsCurrentUserFileActivityRole:
+        return a._isCurrentUserFileActivity;
+    case ThumbnailRole: {
+        if (a._type == Activity::NotificationType && !a._talkNotificationData.userAvatar.isEmpty()) {
+            return generateAvatarThumbnailMap(a._talkNotificationData.userAvatar);
+        }
+
+        if(a._previews.empty()) {
+            return {};
+        }
+
+        const auto preview = a._previews[0];
+        return(generatePreviewMap(preview));
+    }
+    case TalkNotificationConversationTokenRole:
+        return a._talkNotificationData.conversationToken;
+    case TalkNotificationMessageIdRole:
+        return a._talkNotificationData.messageId;
+    case TalkNotificationMessageSentRole:
+        return replyMessageSent(a);
+    case TalkNotificationUserAvatarRole:
+        return a._talkNotificationData.userAvatar;
     default:
         return QVariant();
     }
     return QVariant();
 }
 
-int ActivityListModel::rowCount(const QModelIndex &) const
+int ActivityListModel::rowCount(const QModelIndex &parent) const
 {
+    if(parent.isValid()) {
+        return 0;
+    }
+
     return _finalList.count();
 }
 
 bool ActivityListModel::canFetchMore(const QModelIndex &) const
 {
     // We need to be connected to be able to fetch more
-    if (_accountState && _accountState->isConnected()) {
+    if (_accountState && _accountState->isConnected() && Systray::instance()->isOpen()) {
         // If the fetching is reported to be done or we are currently fetching we can't fetch more
-        if (!_doneFetching && !_currentlyFetching) {
+        if (!_doneFetching && !currentlyFetching()) {
             return true;
         }
     }
@@ -289,7 +388,7 @@ bool ActivityListModel::canFetchMore(const QModelIndex &) const
 
 void ActivityListModel::startFetchJob()
 {
-    if (!_accountState->isConnected()) {
+    if (!_accountState->isConnected() || currentlyFetching()) {
         return;
     }
     auto *job = new JsonApiJob(_accountState->account(), QLatin1String("ocs/v2.php/apps/activity/api/v2/activity"), this);
@@ -297,86 +396,42 @@ void ActivityListModel::startFetchJob()
         this, &ActivityListModel::activitiesReceived);
 
     QUrlQuery params;
+    params.addQueryItem(QLatin1String("previews"), QLatin1String("true"));
     params.addQueryItem(QLatin1String("since"), QString::number(_currentItem));
     params.addQueryItem(QLatin1String("limit"), QString::number(50));
     job->addQueryParams(params);
 
-    _currentlyFetching = true;
+    setAndRefreshCurrentlyFetching(true);
     qCInfo(lcActivity) << "Start fetching activities for " << _accountState->account()->displayName();
     job->start();
 }
 
-void ActivityListModel::activitiesReceived(const QJsonDocument &json, int statusCode)
+void ActivityListModel::setFinalList(const ActivityList &finalList)
 {
-    auto activities = json.object().value("ocs").toObject().value("data").toArray();
+    _finalList = finalList;
+}
 
+const ActivityList &ActivityListModel::finalList() const
+{
+    return _finalList;
+}
+
+int ActivityListModel::currentItem() const
+{
+    return _currentItem;
+}
+
+void ActivityListModel::ingestActivities(const QJsonArray &activities)
+{
     ActivityList list;
-    auto ast = _accountState;
-    if (!ast) {
-        return;
-    }
-
-    if (activities.size() == 0) {
-        _doneFetching = true;
-    }
-
-    _currentlyFetching = false;
 
     QDateTime oldestDate = QDateTime::currentDateTime();
-    oldestDate = oldestDate.addDays(_maxActivitiesDays * -1);
+    oldestDate = oldestDate.addDays(static_cast<qint64>(_maxActivitiesDays) * -1);
 
-    foreach (auto activ, activities) {
-        auto json = activ.toObject();
+    for (const auto &activ : activities) {
+        const auto json = activ.toObject();
 
-        Activity a;
-        a._type = Activity::ActivityType;
-        a._objectType = json.value(QStringLiteral("object_type")).toString();
-        a._accName = ast->account()->displayName();
-        a._id = json.value(QStringLiteral("activity_id")).toInt();
-        a._fileAction = json.value(QStringLiteral("type")).toString();
-        a._subject = json.value(QStringLiteral("subject")).toString();
-        a._message = json.value(QStringLiteral("message")).toString();
-        a._file = json.value(QStringLiteral("object_name")).toString();
-        a._link = QUrl(json.value(QStringLiteral("link")).toString());
-        a._dateTime = QDateTime::fromString(json.value(QStringLiteral("datetime")).toString(), Qt::ISODate);
-        a._icon = json.value(QStringLiteral("icon")).toString();
-
-        auto richSubjectData = json.value(QStringLiteral("subject_rich")).toArray();
-        Q_ASSERT(richSubjectData.size() > 1);
-
-        if(richSubjectData.size() > 1) {
-            a._subjectRich = richSubjectData[0].toString();
-            auto parameters = richSubjectData[1].toObject();
-            const QRegularExpression subjectRichParameterRe(QStringLiteral("({[a-zA-Z0-9]*})"));
-            const QRegularExpression subjectRichParameterBracesRe(QStringLiteral("[{}]"));
-
-            for (auto i = parameters.begin(); i != parameters.end(); ++i) {
-                const auto parameterJsonObject = i.value().toObject();
-                const Activity::RichSubjectParameter parameter = {
-                    parameterJsonObject.value(QStringLiteral("type")).toString(),
-                    parameterJsonObject.value(QStringLiteral("id")).toString(),
-                    parameterJsonObject.value(QStringLiteral("name")).toString(),
-                    parameterJsonObject.contains(QStringLiteral("path")) ? parameterJsonObject.value(QStringLiteral("path")).toString() : QString(),
-                    parameterJsonObject.contains(QStringLiteral("link")) ? QUrl(parameterJsonObject.value(QStringLiteral("link")).toString()) : QUrl(),
-                };
-
-                a._subjectRichParameters[i.key()] = parameter;
-            }
-
-            auto displayString = a._subjectRich;
-            auto i = subjectRichParameterRe.globalMatch(displayString);
-
-            while (i.hasNext()) {
-                const auto match = i.next();
-                auto word = match.captured(1);
-                word.remove(subjectRichParameterBracesRe);
-
-                Q_ASSERT(a._subjectRichParameters.contains(word));
-                displayString = displayString.replace(match.captured(1), a._subjectRichParameters[word].name);
-            }
-
-            a._subjectDisplay = displayString;
-        }
+        auto a = Activity::fromActivityJson(json, _accountState->account());
 
         list.append(a);
         _currentItem = list.last()._id;
@@ -392,9 +447,103 @@ void ActivityListModel::activitiesReceived(const QJsonDocument &json, int status
 
     _activityLists.append(list);
 
-    emit activityJobStatusCode(statusCode);
+    if (list.size() > 0) {
+        std::sort(list.begin(), list.end());
+        beginInsertRows({}, _finalList.size(), _finalList.size() + list.size() - 1);
+        _finalList.append(list);
+        endInsertRows();
 
-    combineActivityLists();
+        appendMoreActivitiesAvailableEntry();
+    }
+}
+
+void ActivityListModel::appendMoreActivitiesAvailableEntry()
+{
+    const QString moreActivitiesEntryObjectType = QLatin1String("activity_fetch_more_activities");
+    if (_showMoreActivitiesAvailableEntry && !_finalList.isEmpty()
+        && _finalList.last()._objectType != moreActivitiesEntryObjectType) {
+        Activity a;
+        a._type = Activity::ActivityType;
+        a._accName = _accountState->account()->displayName();
+        a._id = -1;
+        a._objectType = moreActivitiesEntryObjectType;
+        a._subject = tr("For more activities please open the Activity app.");
+        a._dateTime = QDateTime::currentDateTime();
+
+        if (const auto *app = _accountState->findApp(QLatin1String("activity"))) {
+            a._link = app->url();
+        }
+
+        beginInsertRows({}, _finalList.size(), _finalList.size());
+        _finalList.append(a);
+        endInsertRows();
+    }
+}
+
+void ActivityListModel::insertOrRemoveDummyFetchingActivity()
+{
+    const QString dummyFetchingActivityObjectType = QLatin1String("dummy_fetching_activity");
+    if (_currentlyFetching && _finalList.isEmpty()) {
+        Activity a;
+        a._type = Activity::ActivityType;
+        a._accName = _accountState->account()->displayName();
+        a._id = -2;
+        a._objectType = dummyFetchingActivityObjectType;
+        a._subject = tr("Fetching activities …");
+        a._dateTime = QDateTime::currentDateTime();
+        a._icon = QLatin1String("qrc:///client/theme/colored/change-bordered.svg");
+
+        beginInsertRows({}, 0, 0);
+        _finalList.prepend(a);
+        endInsertRows();
+    } else if (!_finalList.isEmpty() && _finalList.first()._objectType == dummyFetchingActivityObjectType) {
+        beginRemoveRows({}, 0, 0);
+        _finalList.removeAt(0);
+        endRemoveRows();
+    }
+}
+
+void ActivityListModel::clearActivities()
+{
+    _activityLists.clear();
+    if (!_finalList.isEmpty()) {
+        const auto firstActivityIt = std::find_if(std::begin(_finalList), std::end(_finalList),
+            [&](const Activity &activity) { return activity._type == Activity::ActivityType; });
+
+        if (firstActivityIt != std::end(_finalList)) {
+            const auto lastActivityItReverse = std::find_if(std::rbegin(_finalList), std::rend(_finalList),
+                    [&](const Activity &activity) { return activity._type == Activity::ActivityType; });
+
+            const auto lastActivityIt = (lastActivityItReverse + 1).base();
+
+            if (lastActivityIt != std::end(_finalList)) {
+                const int beginRemoveIndex = std::distance(std::begin(_finalList), firstActivityIt);
+                const int endRemoveIndex = std::distance(std::begin(_finalList), lastActivityIt);
+                beginRemoveRows({}, beginRemoveIndex, endRemoveIndex);
+                _finalList.erase(firstActivityIt, std::end(_finalList));
+                endRemoveRows();
+            }
+        }
+    }
+}
+
+void ActivityListModel::activitiesReceived(const QJsonDocument &json, int statusCode)
+{
+    const auto activities = json.object().value(QStringLiteral("ocs")).toObject().value(QStringLiteral("data")).toArray();
+
+    if (!_accountState) {
+        return;
+    }
+
+    if (activities.empty()) {
+        _doneFetching = true;
+    }
+
+    setAndRefreshCurrentlyFetching(false);
+
+    ingestActivities(activities);
+
+    emit activityJobStatusCode(statusCode);
 }
 
 void ActivityListModel::addErrorToActivityList(Activity activity)
@@ -464,8 +613,15 @@ void ActivityListModel::removeActivityFromActivityList(Activity activity)
     int index = -1;
     if (activity._type == Activity::ActivityType) {
         index = _activityLists.indexOf(activity);
-        if (index != -1)
+        if (index != -1) {
             _activityLists.removeAt(index);
+            const auto indexInFinalList = _finalList.indexOf(activity);
+            if (indexInFinalList != -1) {
+                beginRemoveRows({}, indexInFinalList, indexInFinalList);
+                _finalList.removeAt(indexInFinalList);
+                endRemoveRows();
+            }
+        }
     } else if (activity._type == Activity::NotificationType) {
         index = _notificationLists.indexOf(activity);
         if (index != -1)
@@ -483,7 +639,7 @@ void ActivityListModel::removeActivityFromActivityList(Activity activity)
     }
 }
 
-void ActivityListModel::triggerDefaultAction(int activityIndex)
+void ActivityListModel::slotTriggerDefaultAction(const int activityIndex)
 {
     if (activityIndex < 0 || activityIndex >= _finalList.size()) {
         qCWarning(lcActivity) << "Couldn't trigger default action at index" << activityIndex << "/ final list size:" << _finalList.size();
@@ -491,7 +647,7 @@ void ActivityListModel::triggerDefaultAction(int activityIndex)
     }
 
     const auto modelIndex = index(activityIndex);
-    const auto path = data(modelIndex, PathRole).toUrl();
+    const auto path = data(modelIndex, PathRole).toString();
 
     const auto activity = _finalList.at(activityIndex);
     if (activity._status == SyncFileItem::Conflict) {
@@ -536,20 +692,24 @@ void ActivityListModel::triggerDefaultAction(int activityIndex)
         connect(_currentInvalidFilenameDialog, &InvalidFilenameDialog::accepted, folder, [folder]() {
             folder->scheduleThisFolderSoon();
         });
+        connect(_currentInvalidFilenameDialog, &InvalidFilenameDialog::acceptedInvalidName, folder, [folder](const QString& filePath) {
+            folder->acceptInvalidFileName(filePath);
+            folder->scheduleThisFolderSoon();
+        });
         _currentInvalidFilenameDialog->open();
         ownCloudGui::raiseDialog(_currentInvalidFilenameDialog);
         return;
     }
 
-    if (path.isValid()) {
-        QDesktopServices::openUrl(path);
+    if (!path.isEmpty()) {
+        QDesktopServices::openUrl(QUrl::fromLocalFile(path));
     } else {
         const auto link = data(modelIndex, LinkRole).toUrl();
         Utility::openBrowser(link);
     }
 }
 
-void ActivityListModel::triggerAction(int activityIndex, int actionIndex)
+void ActivityListModel::slotTriggerAction(const int activityIndex, const int actionIndex)
 {
     if (activityIndex < 0 || activityIndex >= _finalList.size()) {
         qCWarning(lcActivity) << "Couldn't trigger action on activity at index" << activityIndex << "/ final list size:" << _finalList.size();
@@ -573,11 +733,98 @@ void ActivityListModel::triggerAction(int activityIndex, int actionIndex)
     emit sendNotificationRequest(activity._accName, action._link, action._verb, activityIndex);
 }
 
+void ActivityListModel::slotTriggerDismiss(const int activityIndex)
+{
+    if (activityIndex < 0 || activityIndex >= _finalList.size()) {
+        qCWarning(lcActivity) << "Couldn't trigger action on activity at index" << activityIndex << "/ final list size:" << _finalList.size();
+        return;
+    }
+
+    const auto activityLinks = _finalList[activityIndex]._links;
+
+    const auto foundActivityLinkIt = std::find_if(std::cbegin(activityLinks), std::cend(activityLinks), [](const ActivityLink &link) {
+        return link._verb == QStringLiteral("DELETE");
+    });
+
+    if (foundActivityLinkIt == std::cend(activityLinks)) {
+        qCWarning(lcActivity) << "Couldn't find dismiss action in activity at index" << activityIndex
+                              << " links.size() " << activityLinks.size();
+        return;
+    }
+
+    const auto actionIndex = static_cast<int>(std::distance(activityLinks.begin(), foundActivityLinkIt));
+
+    if (actionIndex < 0 || actionIndex > activityLinks.size()) {
+        qCWarning(lcActivity) << "Couldn't find dismiss action in activity at index" << activityIndex
+                              << " actionIndex found " << actionIndex;
+        return;
+    }
+
+    slotTriggerAction(activityIndex, actionIndex);
+}
+
 AccountState *ActivityListModel::accountState() const
 {
     return _accountState;
 }
 
+QVariantList ActivityListModel::convertLinksToActionButtons(const Activity &activity)
+{
+    QVariantList customList;
+
+    if (activity._links.size() == 1) {
+        return customList;
+    }
+
+    if (static_cast<quint32>(activity._links.size()) > maxActionButtons()) {
+        customList << ActivityListModel::convertLinkToActionButton(activity._links.first());
+        return customList;
+    }
+
+    for (const auto &activityLink : activity._links) {
+        if (activityLink._primary
+            || activityLink._verb == QStringLiteral("DELETE")
+            || activityLink._verb == QStringLiteral("WEB")) {
+            customList << ActivityListModel::convertLinkToActionButton(activityLink);
+        }
+    }
+
+    return customList;
+}
+
+QVariant ActivityListModel::convertLinkToActionButton(const OCC::ActivityLink &activityLink)
+{
+    auto activityLinkCopy = activityLink;
+
+    const auto isReplyIconApplicable = activityLink._verb == QStringLiteral("REPLY");
+
+    const QString replyButtonPath = QStringLiteral("image://svgimage-custom-color/reply.svg");
+
+    if (isReplyIconApplicable) {
+        activityLinkCopy._imageSource = QString(replyButtonPath + "/");
+        activityLinkCopy._imageSourceHovered = QString(replyButtonPath + "/");
+    }
+
+    return QVariant::fromValue(activityLinkCopy);
+}
+
+QVariantList ActivityListModel::convertLinksToMenuEntries(const Activity &activity)
+{
+    QVariantList customList;
+
+    if (static_cast<quint32>(activity._links.size()) > maxActionButtons()) {
+        for (int i = 0; i < activity._links.size(); ++i) {
+            const auto &activityLink = activity._links[i];
+            if (!activityLink._primary) {
+                customList << QVariantMap{
+                    {QStringLiteral("actionIndex"), i}, {QStringLiteral("label"), activityLink._label}};
+            }
+        }
+    }
+
+    return customList;
+}
+
 void ActivityListModel::combineActivityLists()
 {
     ActivityList resultList;
@@ -602,32 +849,20 @@ void ActivityListModel::combineActivityLists()
     if (_activityLists.count() > 0) {
         std::sort(_activityLists.begin(), _activityLists.end());
         resultList.append(_activityLists);
-
-        if(_showMoreActivitiesAvailableEntry) {
-            Activity a;
-            a._type = Activity::ActivityType;
-            a._accName = _accountState->account()->displayName();
-            a._id = -1;
-            a._subject = tr("For more activities please open the Activity app.");
-            a._dateTime = QDateTime::currentDateTime();
-
-            AccountApp *app = _accountState->findApp(QLatin1String("activity"));
-            if(app) {
-                a._link = app->url();
-            }
-
-            resultList.append(a);
-        }
     }
 
-    beginResetModel();
-    _finalList.clear();
-    endResetModel();
-
-    if (resultList.count() > 0) {
-        beginInsertRows(QModelIndex(), 0, resultList.count() - 1);
+    if (_finalList.isEmpty() && !resultList.isEmpty()) {
+        beginInsertRows({}, 0, resultList.size() - 1);
         _finalList = resultList;
         endInsertRows();
+    } else if (!_finalList.isEmpty()) {
+        beginResetModel();
+        _finalList = resultList;
+        endResetModel();
+    }
+
+    if (_activityLists.size() > 0) {
+        appendMoreActivitiesAvailableEntry();
     }
 }
 
@@ -640,15 +875,12 @@ void ActivityListModel::fetchMore(const QModelIndex &)
 {
     if (canFetchActivities()) {
         startFetchJob();
-    } else {
-        _doneFetching = true;
-        combineActivityLists();
     }
 }
 
 void ActivityListModel::slotRefreshActivity()
 {
-    _activityLists.clear();
+    clearActivities();
     _doneFetching = false;
     _currentItem = 0;
     _totalActivitiesFetched = 0;
@@ -662,14 +894,38 @@ void ActivityListModel::slotRefreshActivity()
     }
 }
 
+void ActivityListModel::slotRefreshActivityInitial()
+{
+    if (_activityLists.isEmpty() && !currentlyFetching()) {
+        slotRefreshActivity();
+    }
+}
+
 void ActivityListModel::slotRemoveAccount()
 {
     _finalList.clear();
     _activityLists.clear();
-    _currentlyFetching = false;
+    setAndRefreshCurrentlyFetching(false);
     _doneFetching = false;
     _currentItem = 0;
     _totalActivitiesFetched = 0;
     _showMoreActivitiesAvailableEntry = false;
 }
+
+void ActivityListModel::setReplyMessageSent(const int activityIndex, const QString &message)
+{
+    if (activityIndex < 0 || activityIndex >= _finalList.size()) {
+        qCWarning(lcActivity) << "Couldn't trigger action on activity at index" << activityIndex << "/ final list size:" << _finalList.size();
+        return;
+    }
+
+    _finalList[activityIndex]._talkNotificationData.messageSent = message;
+
+    emit dataChanged(index(activityIndex, 0), index(activityIndex, 0), {ActivityListModel::TalkNotificationMessageSentRole});
+}
+
+QString ActivityListModel::replyMessageSent(const Activity &activity) const
+{
+    return activity._talkNotificationData.messageSent;
+}
 }
