@@ -17,6 +17,7 @@ public:
 
 signals:
     void newNotificationList(ActivityList);
+    void newIncomingCallsList(ActivityList);
 
 public slots:
     void slotFetchNotifications();
@@ -24,7 +25,6 @@ public slots:
 private slots:
     void slotNotificationsReceived(const QJsonDocument &json, int statusCode);
     void slotEtagResponseHeaderReceived(const QByteArray &value, int statusCode);
-    void slotAllowDesktopNotificationsChanged(bool isAllowed);
 
 private:
     QPointer<JsonApiJob> _notificationJob;
