@@ -43,9 +43,14 @@ OwncloudAdvancedSetupPage::OwncloudAdvancedSetupPage(OwncloudWizard *wizard)
     , _ocWizard(wizard)
 {
     _ui.setupUi(this);
-
     setupResoultionWidget();
 
+    _filePathLabel.reset(new ElidedLabel);
+    _filePathLabel->setElideMode(Qt::ElideMiddle);
+    _filePathLabel->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
+    _filePathLabel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
+    _ui.locationsGridLayout->addWidget(_filePathLabel.data(), 3, 3);
+
     registerField(QLatin1String("OCSyncFromScratch"), _ui.cbSyncFromScratch);
 
     auto sizePolicy = _progressIndi->sizePolicy();
@@ -57,7 +62,13 @@ OwncloudAdvancedSetupPage::OwncloudAdvancedSetupPage(OwncloudWizard *wizard)
     setupCustomization();
 
     connect(_ui.pbSelectLocalFolder, &QAbstractButton::clicked, this, &OwncloudAdvancedSetupPage::slotSelectFolder);
-    setButtonText(QWizard::NextButton, tr("Connect"));
+    setButtonText(QWizard::FinishButton, tr("Connect"));
+
+    if (Theme::instance()->enforceVirtualFilesSyncFolder()) {
+        _ui.rSyncEverything->setDisabled(true);
+        _ui.rSelectiveSync->setDisabled(true);
+        _ui.bSelectiveSync->setDisabled(true);
+    }
 
     connect(_ui.rSyncEverything, &QAbstractButton::clicked, this, &OwncloudAdvancedSetupPage::slotSyncEverythingClicked);
     connect(_ui.rSelectiveSync, &QAbstractButton::clicked, this, &OwncloudAdvancedSetupPage::slotSelectiveSyncClicked);
@@ -110,6 +121,7 @@ void OwncloudAdvancedSetupPage::setupCustomization()
     variant = theme->customMedia(Theme::oCSetupBottom);
     WizardCommon::setupCustomMedia(variant, _ui.bottomLabel);
 
+    WizardCommon::customizeHintLabel(_filePathLabel.data());
     WizardCommon::customizeHintLabel(_ui.lFreeSpace);
     WizardCommon::customizeHintLabel(_ui.lSyncEverythingSizeLabel);
     WizardCommon::customizeHintLabel(_ui.lSelectiveSyncSizeLabel);
@@ -134,8 +146,8 @@ void OwncloudAdvancedSetupPage::initializePage()
     }
 
     _checking = false;
-    _ui.lSelectiveSyncSizeLabel->setText(QString());
-    _ui.lSyncEverythingSizeLabel->setText(QString());
+    _ui.lSelectiveSyncSizeLabel->clear();
+    _ui.lSyncEverythingSizeLabel->clear();
 
     // Update the local folder - this is not guaranteed to find a good one
     QString goodLocalFolder = FolderMan::instance()->findGoodPathForNewSyncFolder(localFolder(), serverUrl());
@@ -145,7 +157,7 @@ void OwncloudAdvancedSetupPage::initializePage()
     updateStatus();
 
     // ensure "next" gets the focus, not obSelectLocalFolder
-    QTimer::singleShot(0, wizard()->button(QWizard::NextButton), SLOT(setFocus()));
+    QTimer::singleShot(0, wizard()->button(QWizard::FinishButton), qOverload<>(&QWidget::setFocus));
 
     auto acc = static_cast<OwncloudWizard *>(wizard())->account();
     auto quotaJob = new PropfindJob(acc, _remoteFolder, this);
@@ -168,7 +180,7 @@ void OwncloudAdvancedSetupPage::initializePage()
     _ui.confCheckBoxExternal->setChecked(cfgFile.confirmExternalStorage());
 
     fetchUserAvatar();
-    fetchUserData();
+    setUserInformation();
 
     customizeStyle();
 
@@ -182,10 +194,11 @@ void OwncloudAdvancedSetupPage::fetchUserAvatar()
 {
     // Reset user avatar
     const auto appIcon = Theme::instance()->applicationIcon();
-    _ui.lServerIcon->setPixmap(appIcon.pixmap(48));
+    // To match the folder icon opposite the avatar -- that is 60x60, minus padding
+    _ui.lServerIcon->setPixmap(appIcon.pixmap(32));
     // Fetch user avatar
     const auto account = _ocWizard->account();
-    auto avatarSize = 64;
+    auto avatarSize = 32;
     if (Theme::isHidpi()) {
         avatarSize *= 2;
     }
@@ -201,20 +214,9 @@ void OwncloudAdvancedSetupPage::fetchUserAvatar()
     avatarJob->start();
 }
 
-void OwncloudAdvancedSetupPage::fetchUserData()
+void OwncloudAdvancedSetupPage::setUserInformation()
 {
     const auto account = _ocWizard->account();
-
-    // Fetch user data
-    const auto userJob = new JsonApiJob(account, QLatin1String("ocs/v1.php/cloud/user"), this);
-    userJob->setTimeout(20 * 1000);
-    connect(userJob, &JsonApiJob::jsonReceived, this, [this](const QJsonDocument &json) {
-        const auto objData = json.object().value("ocs").toObject().value("data").toObject();
-        const auto displayName = objData.value("display-name").toString();
-        _ui.userNameLabel->setText(displayName);
-    });
-    userJob->start();
-
     const auto serverUrl = account->url().toString();
     setServerAddressLabelUrl(serverUrl);
     const auto userName = account->davDisplayName();
@@ -261,16 +263,14 @@ void OwncloudAdvancedSetupPage::updateStatus()
 
     QString t;
 
-    setLocalFolderPushButtonPath(locFolder);
-
     if (dataChanged()) {
         if (_remoteFolder.isEmpty() || _remoteFolder == QLatin1String("/")) {
             t = "";
         } else {
-            t = Utility::escape(tr("%1 folder '%2' is synced to local folder '%3'")
+            t = Utility::escape(tr(R"(%1 folder "%2" is synced to local folder "%3")")
                                     .arg(Theme::instance()->appName(), _remoteFolder,
                                         QDir::toNativeSeparators(locFolder)));
-            _ui.rSyncEverything->setText(tr("Sync the folder '%1'").arg(_remoteFolder));
+            _ui.rSyncEverything->setText(tr("Sync the folder \"%1\"").arg(_remoteFolder));
         }
 
         const bool dirNotEmpty(QDir(locFolder).entryList(QDir::AllEntries | QDir::NoDotAndDotDot).count() > 0);
@@ -282,6 +282,8 @@ void OwncloudAdvancedSetupPage::updateStatus()
         setResolutionGuiVisible(false);
     }
 
+    _filePathLabel->setText(QDir::toNativeSeparators(locFolder));
+
     QString lfreeSpaceStr = Utility::octetsToString(availableLocalSpace());
     _ui.lFreeSpace->setText(QString(tr("%1 free space", "%1 gets replaced with the size and a matching unit. Example: 3 MB or 5 GB")).arg(lfreeSpaceStr));
 
@@ -338,7 +340,8 @@ QUrl OwncloudAdvancedSetupPage::serverUrl() const
 
 int OwncloudAdvancedSetupPage::nextId() const
 {
-    return WizardCommon::Page_Result;
+    // tells the caller that this is the last dialog page
+    return -1;
 }
 
 QString OwncloudAdvancedSetupPage::localFolder() const
@@ -432,7 +435,6 @@ void OwncloudAdvancedSetupPage::slotSelectFolder()
         // TODO: remove when UX decision is made
         refreshVirtualFilesAvailibility(dir);
 
-        setLocalFolderPushButtonPath(dir);
         wizard()->setProperty("localFolder", dir);
         updateStatus();
     }
@@ -442,22 +444,6 @@ void OwncloudAdvancedSetupPage::slotSelectFolder()
     setErrorString(errorStr);
 }
 
-
-void OwncloudAdvancedSetupPage::setLocalFolderPushButtonPath(const QString &path)
-{
-    const auto homeDir = QDir::homePath().endsWith('/') ? QDir::homePath() : QDir::homePath() + QLatin1Char('/');
-
-    if (!path.startsWith(homeDir)) {
-        _ui.pbSelectLocalFolder->setText(QDir::toNativeSeparators(path));
-        return;
-    }
-
-    auto prettyPath = path;
-    prettyPath.remove(0, homeDir.size());
-
-    _ui.pbSelectLocalFolder->setText(QDir::toNativeSeparators(prettyPath));
-}
-
 void OwncloudAdvancedSetupPage::slotSelectiveSyncClicked()
 {
     AccountPtr acc = static_cast<OwncloudWizard *>(wizard())->account();
