@@ -290,7 +290,10 @@ void BulkPropagatorJob::slotStartUpload(SyncFileItemPtr item,
     const QString originalFilePath = propagator()->fullLocalPath(item->_file);
 
     if (!FileSystem::fileExists(fullFilePath)) {
-        return slotOnErrorStartFolderUnlock(item, SyncFileItem::SoftError, tr("File Removed (start upload) %1").arg(fullFilePath));
+        _pendingChecksumFiles.remove(item->_file);
+        slotOnErrorStartFolderUnlock(item, SyncFileItem::SoftError, tr("File Removed (start upload) %1").arg(fullFilePath));
+        checkPropagationIsDone();
+        return;
     }
     const time_t prevModtime = item->_modtime; // the _item value was set in PropagateUploadFile::start()
     // but a potential checksum calculation could have taken some time during which the file could
@@ -299,7 +302,7 @@ void BulkPropagatorJob::slotStartUpload(SyncFileItemPtr item,
     item->_modtime = FileSystem::getModTime(originalFilePath);
     if (item->_modtime <= 0) {
         _pendingChecksumFiles.remove(item->_file);
-        slotOnErrorStartFolderUnlock(item, SyncFileItem::NormalError, tr("File %1 has invalid modified time. Do not upload to the server.").arg(QDir::toNativeSeparators(item->_file)));
+        slotOnErrorStartFolderUnlock(item, SyncFileItem::NormalError, tr("File %1 has invalid modification time. Do not upload to the server.").arg(QDir::toNativeSeparators(item->_file)));
         checkPropagationIsDone();
         return;
     }
@@ -388,6 +391,8 @@ void BulkPropagatorJob::slotPutFinishedOneFile(const BulkUploadItem &singleFile,
     computeFileId(singleFile._item, fileReply);
 
     singleFile._item->_etag = etag;
+    singleFile._item->_fileId = getHeaderFromJsonReply(fileReply, "fileid");
+    singleFile._item->_remotePerm = RemotePermissions::fromServerString(getHeaderFromJsonReply(fileReply, "permissions"));
 
     if (getHeaderFromJsonReply(fileReply, "X-OC-MTime") != "accepted") {
         // X-OC-MTime is supported since owncloud 5.0.   But not when chunking.
@@ -404,12 +409,18 @@ void BulkPropagatorJob::slotPutFinished()
 
     slotJobDestroyed(job); // remove it from the _jobs list
 
+    const auto jobError = job->reply()->error();
+
     const auto replyData = job->reply()->readAll();
     const auto replyJson = QJsonDocument::fromJson(replyData);
     const auto fullReplyObject = replyJson.object();
 
     for (const auto &singleFile : _filesToUpload) {
         if (!fullReplyObject.contains(singleFile._remotePath)) {
+            if (jobError != QNetworkReply::NoError) {
+                singleFile._item->_status = SyncFileItem::NormalError;
+                abortWithError(singleFile._item, SyncFileItem::NormalError, tr("Network error: %1").arg(jobError));
+            }
             continue;
         }
         const auto singleReplyObject = fullReplyObject[singleFile._remotePath].toObject();
@@ -664,7 +675,7 @@ void BulkPropagatorJob::handleFileRestoration(SyncFileItemPtr item,
             || item->_status == SyncFileItem::Conflict) {
             item->_status = SyncFileItem::Restoration;
         } else {
-            item->_errorString += tr("; Restoration Failed: %1").arg(errorString);
+            item->_errorString += tr("Restoration failed: %1").arg(errorString);
         }
     } else {
         if (item->_errorString.isEmpty()) {
