@@ -1238,9 +1238,9 @@ bool ClientSideEncryption::newMnemonicGenerated() const
 }
 
 void ClientSideEncryption::decryptPrivateKey(const AccountPtr &account, const QByteArray &key) {
-    QString msg = tr("Please enter your end to end encryption passphrase:<br>"
+    QString msg = tr("Please enter your End-to-End encryption passphrase:<br>"
                      "<br>"
-                     "User: %2<br>"
+                     "Username: %2<br>"
                      "Account: %3<br>")
                       .arg(Utility::escape(account->credentials()->user()),
                            Utility::escape(account->displayName()));
@@ -1656,7 +1656,6 @@ bool EncryptionHelper::fileEncryption(const QByteArray &key, const QByteArray &i
 
     QByteArray out(blockSize + OCC::Constants::e2EeTagSize - 1, '\0');
     int len = 0;
-    int total_len = 0;
 
     qCDebug(lcCse) << "Starting to encrypt the file" << input->fileName() << input->atEnd();
     while(!input->atEnd()) {
@@ -1673,7 +1672,6 @@ bool EncryptionHelper::fileEncryption(const QByteArray &key, const QByteArray &i
         }
 
         output->write(out, len);
-        total_len += len;
     }
 
     if(1 != EVP_EncryptFinal_ex(ctx, unsignedData(out), &len)) {
@@ -1681,7 +1679,6 @@ bool EncryptionHelper::fileEncryption(const QByteArray &key, const QByteArray &i
         return false;
     }
     output->write(out, len);
-    total_len += len;
 
     /* Get the e2EeTag */
     QByteArray e2EeTag(OCC::Constants::e2EeTagSize, '\0');
@@ -1865,7 +1862,6 @@ QByteArray EncryptionHelper::StreamingDecryptor::chunkDecryption(const char *inp
         return QByteArray();
     }
 
-    qint64 bytesWritten = 0;
     qint64 inputPos = 0;
 
     QByteArray decryptedBlock(blockSize + OCC::Constants::e2EeTagSize - 1, '\0');
@@ -1894,8 +1890,6 @@ QByteArray EncryptionHelper::StreamingDecryptor::chunkDecryption(const char *inp
             return QByteArray();
         }
 
-        bytesWritten += writtenToOutput;
-
         // advance input position for further read
         inputPos += encryptedBlock.size();
 
@@ -1934,8 +1928,6 @@ QByteArray EncryptionHelper::StreamingDecryptor::chunkDecryption(const char *inp
             return QByteArray();
         }
 
-        bytesWritten += writtenToOutput;
-
         _decryptedSoFar += OCC::Constants::e2EeTagSize;
 
         _isFinished = true;
