@@ -958,12 +958,12 @@ private slots:
             fakeFolder.syncOnce();
         }
 
-        QVERIFY(!fakeFolder.currentLocalState().find(src));
-        QVERIFY(fakeFolder.currentLocalState().find(getName(dest)));
+        QVERIFY(fakeFolder.currentLocalState().find(getName(src)));
+        QVERIFY(!fakeFolder.currentLocalState().find(getName(dest)));
         if (vfsMode == Vfs::WithSuffix)
         {
             // the placeholder was not restored as it is still in error state
-            QVERIFY(!fakeFolder.currentLocalState().find(dest));
+            QVERIFY(!fakeFolder.currentLocalState().find(getName(dest)));
         }
         QVERIFY(fakeFolder.currentRemoteState().find(src));
         QVERIFY(!fakeFolder.currentRemoteState().find(dest));
